from __future__ import annotations
import typing as t

from flask import (
    Flask as _Flask,
    Blueprint as FlaskBlueprint,
    current_app,
    redirect,
    render_template,
    url_for,
    session,
)

from jinja2 import BaseLoader, ChoiceLoader, PrefixLoader, Template

from flask_multipass import IdentityInfo


class Blueprint(FlaskBlueprint):
    def real_template_name(
        self, template: str | Template | list[str | Template]
    ) -> str:
        if self.jinja_loader is not None:
            # XXX This currently assumes ":" as PrefixLoader.delimiter in the app
            if isinstance(template, str) and ":" not in template:
                template = self.name + ":" + template
            elif isinstance(template, list):
                for i, tpl in enumerate(template):
                    if isinstance(t, str) and ":" not in tpl:
                        template[i] = self.name + ":" + tpl
        return template

    def render_template(
        self,
        template: str | Template | list[str | Template],
        **context: t.Any,
    ) -> str:
        template = self.real_template_name(template)
        return render_template(template, **context)


class Flask(_Flask):
    def create_global_jinja_loader(self) -> BaseLoader:
        return ChoiceLoader(
            [
                PrefixLoader({}, delimiter=":"),
                self.jinja_loader,
            ]
        )

    def register_blueprint(self, blueprint: Blueprint, **options: t.Any) -> None:
        super().register_blueprint(blueprint, **options)
        if blueprint.jinja_loader is not None:
            self.jinja_env.loader.loaders[0].mapping[blueprint.name] = (
                blueprint.jinja_loader
            )


_sentinel = object()


def get_template_attribute(
    template_name: str,
    attribute: str,
    default: t.Any = _sentinel,
    vars: t.Dict[str, t.Any] | None = None,
    shared: bool = False,
    locals: t.Mapping[str, t.Any] | None = None,
) -> t.Any:
    """Loads a macro (or variable) a template exports.  This can be used to
    invoke a macro from within Python code.  If you for example have a
    template named :file:`_cider.html` with the following contents:

    .. sourcecode:: html+jinja

       {% macro hello(name) %}Hello {{ name }}!{% endmacro %}

    You can access this from Python code like this::

        hello = get_template_attribute('_cider.html', 'hello')
        return hello('World')

    .. versionadded:: 0.2

    :param template_name: the name of the template
    :param attribute: the name of the variable of macro to access
    """
    mod = current_app.jinja_env.get_template(template_name).make_module(
        vars, shared, locals
    )
    if default is _sentinel:
        return getattr(mod, attribute)
    else:
        return getattr(mod, attribute, default)


def identity_handler(identity_info: IdentityInfo):
    session["identity"] = dict(
        identifier=identity_info.identifier,
        provider=identity_info.provider.name,
        secure_login=identity_info.secure_login,
        data=identity_info.data,
    )


def require_login():
    if "identity" not in session:
        return redirect(url_for("login"))
