from . import db, multipass
from .instance import list_templates, list_images
from .models import Schild, SchildForm, PrintForm

from flask import (
    Blueprint,
    current_app as app,
    flash,
    render_template,
    request,
    redirect,
    url_for,
)

import flask_weasyprint

from pyipp import IPP
from pyipp.enums import IppOperation
from pyipp.exceptions import IPPError

bp = Blueprint("views", __name__)


@bp.route("/")
def index():
    schilder = db.session.execute(db.select(Schild)).scalars()
    return render_template("index.html.j2", schilder=schilder)


def _do_schild_form(schild):
    form = SchildForm(request.form, obj=schild)
    form.image.choices = list(list_images())
    form.template.choices = list(list_templates())
    if request.method == "POST" and form.validate():
        form.populate_obj(schild)
        db.session.add(schild)
        db.session.commit()
    return form


@bp.route("/schild/<uuid:ident>", methods=["GET", "POST"])
def schild(ident):
    schild = db.get_or_404(Schild, ident)
    form = _do_schild_form(schild)
    printform = PrintForm()
    printform.printer.choices = list(app.config["PRINTERS"])
    return render_template(
        "schild.html.j2",
        schild=schild,
        form=form,
        printform=printform,
    )


@bp.route("/schild/<uuid:ident>/print", methods=["POST"])
async def print(ident):
    form = PrintForm(request.form)
    form.printer.choices = list(app.config["PRINTERS"])
    if form.validate():
        async with IPP(app.config["PRINTERS"][form.printer.data]) as ipp:
            try:
                response = await ipp.execute(
                    IppOperation.PRINT_JOB,
                    {
                        "operation-attributes-tag": {
                            "requesting-user-name": "schilder2000",
                            "job-name": f"schild-{ident}.pdf",
                            "document-name": f"schild-{ident}.pdf",
                            "document-format": "application/pdf",
                            "copies": form.copies.data,
                            "job-media-sheets": form.copies.data,
                        },
                        "data": flask_weasyprint.HTML(
                            url_for("instance.schild_html", ident=ident)
                        ).write_pdf(),
                    },
                )
                flash("Druckt …")
                app.logger.debug("print %s: %r", ident, response)
            except IPPError:
                flash("Druck fehlgeschlagen!", "error")
                pass
        return redirect(url_for(".schild", ident=ident))


@bp.route("/create", methods=["GET", "POST"])
def create():
    schild = Schild()
    form = _do_schild_form(schild)
    if request.method == "POST":
        return redirect(url_for(".schild", ident=schild.ident))
    else:
        return render_template(
            "schild.html.j2",
            form=form,
        )


@bp.route("/logout")
def logout():
    return multipass.logout(url_for(".index"), clear_session=True)
